<?php

if ( ! defined( 'ABSPATH' ) ) exit;

class WooCommerce_Shipmate_List_Order_Actions {

	/**
	 * The single instance of WooCommerce_Shipmate_List_Order_Actions.
	 * @var 	object
	 * @access  private
	 * @since 	1.0.0
	 */
	private static $_instance = null;

	/**
	 * The main plugin object.
	 * @var 	object
	 * @access  public
	 * @since 	1.0.0
	 */
	public $parent = null;

	/**
	 * Prefix for plugin settings.
	 * @var     string
	 * @access  public
	 * @since   1.0.0
	 */
	public $base = '';



	/**
	 * The shipmate integration class
	 * @var     WooCommerce_Shipmate_Integration
	 * @access  protected
	 * @since   1.0.0
	 */
	protected $shipmate_integration = null;

	/**
	 * The shipmate service class
	 * @var     WooCommerce_Shipmate_Service
	 * @access  protected
	 * @since   1.0.0
	 */
	protected $shipmate_service = null;

	/**
	 * The key for post_meta to state an order has been printed
	 * @var     WooCommerce_Shipmate_Service
	 * @access  protected
	 * @since   1.0.0
	 */
	protected $printed_meta_key = "_wc_order_printed_with_shipmate";
	protected $tracking_reference_meta_key = "_wc_order_tracking_reference";

	/**
	 * Available settings for plugin.
	 * @var     array
	 * @access  public
	 * @since   1.0.0
	 */
	public $settings = array();

	public function __construct ( $parent ) {

		$this->parent = $parent;

		$this->base = 'wpt_';

		$this->shipmate_integration = new WooCommerce_Shipmate_Integration();
		$this->shipmate_service = new WooCommerce_Shipmate_Service($this->shipmate_integration);

		if(!empty(get_option('shipmate_sender_token', null))) {
			add_filter('bulk_actions-edit-shop_order', array($this,'add_buttons') );	
			add_action('wp_ajax_shipmate_prints', array($this, 'shipmate_prints'));
			add_action('wp_ajax_shipmate_get_orders', array($this, 'shipmate_get_orders'));
		}

	}

	function shipmate_get_orders() {
		//Loop through order ids
		$order_ids = $_POST['order_ids'];


		$data = [];
		foreach($order_ids as $order_id) {
			$disabled = $created_with_shipmate = get_post_meta( $order_id, "_wc_order_printed_with_shipmate", true ) == 'yes';

			$order =  wc_get_order($order_id);

			if(WooCommerce_Shipmate_Integration::version_check('3.0')) {
				$status = $order->get_status();
			} else {
				$status = $order->status;
			}

			$can_print = true;

			//If processing stop is on and the order is no processing then can print = false
			if(get_option("_shipmate_process_block") == 'true' && $status !== "processing") {
	    		$can_print = false;
	    		//If created with shipmate
	    	}
	    	if($created_with_shipmate) {
	    		$can_print = true;
	    	}
			$data[] = [
				'id' => $order_id,
				'disabled' => $disabled,
				'can_print' => $can_print
			];
		}

		echo json_encode($data);
		die();
	}

	
	function shipmate_prints() {


		$order_id = $_POST['order_id'];
		$order =  wc_get_order($order_id);


		$parcelDimensions = [];

		$parcelDimensions['parcel_weights'][] = $_POST['parcel_weight'];
		$parcelDimensions['parcel_lengths'][] = $_POST['parcel_length'];
		$parcelDimensions['parcel_widths'][] = $_POST['parcel_width'];
		$parcelDimensions['parcel_depths'][] = $_POST['parcel_depth'];


		$consignment = new WooCommerce_Shipmate_Consignment($order, 
			$parcelDimensions, 
			!empty($_POST['delivery_service_key']) ? $_POST['delivery_service_key'] : "",
			!empty($_POST['dispatch_date']) ? $_POST['dispatch_date'] : date('Y-m-d', strtotime('now'))
		);
		// $consignment->set_delivery_service_key(!empty($_POST['delivery_service_key']) ? $_POST['delivery_service_key'] : "");

		$wordpress_user = wp_get_current_user();
		$shipmate_user_id = get_user_meta($wordpress_user->ID, '_shipmate_user_id', true);
		$result = $this->shipmate_service->create_consignment($shipmate_user_id, $consignment->toArray());
		
		switch($result->http_code()) {
			case "403":
				$message = sprintf( __( 'Unable to connect to Shipmate (Invalid Access Token).', 'shipmate' ));
	    		$order->add_order_note( $message );
	    		echo(json_encode(array('success' => false)));
			    die();
	    		break;
	    	case "400":
	    		foreach($result->data() as $key => $error) {
	    			$message = sprintf( __( 'Shipmate error: %s'), $error);
	    			$order->add_order_note( $message );
	    		}




	    		if(get_option("_shipmate_status_failed", "Don\'t Change") != "Don\'t Change") {

					if(WooCommerce_Shipmate_Integration::version_check('3.0')) {
						$order->set_status(get_option("_shipmate_status_failed"));
						$order->save();
					} else {
						$order->update_status(get_option("_shipmate_status_failed"));
					}

					
				}
				echo(json_encode(array('success' => false)));
			    die();
	    		break;
	    	case "201":
			    foreach($result->data() as $key => $label) {
			    	// if we haven't previously created these parcels, save the tracking reference and dimensions
			    	if(get_post_meta( $order_id, $this->printed_meta_key, true ) != 'yes') {
				    	$message = sprintf( __( 'Shipping Label generated with Tracking Reference %s for %s. (%s - %s)', 'shipmate' ), $label->tracking_reference, $label->parcel_reference, $label->carrier, $label->service_name );
				    	$order->add_order_note( $message );


				    	

				    	// store the carrier and service used
				    	update_post_meta( $order_id, '_shipmate_carrier', $label->carrier );
				    	update_post_meta( $order_id, '_shipmate_service', $label->service_name );

				    	// store tracking references / dims in meta for the order
				    	add_post_meta($order_id, '_shipmate_parcel_reference', $label->parcel_reference, false);
				    	add_post_meta($order_id, '_shipmate_tracking_reference', $label->tracking_reference, false);
				    	add_post_meta($order_id, '_shipmate_parcel_weight', $parcelDimensions['parcel_weights'][$key], false);
				    	add_post_meta($order_id, '_shipmate_parcel_length', $parcelDimensions['parcel_lengths'][$key], false);
				    	add_post_meta($order_id, '_shipmate_parcel_width', $parcelDimensions['parcel_widths'][$key], false);
				    	add_post_meta($order_id, '_shipmate_parcel_depth', $parcelDimensions['parcel_depths'][$key], false);

				    	if ( function_exists( 'wc_st_add_tracking_number' ) ) {
				    		$provider = $this->shipmate_integration->convert_carrier_to_provider($label->carrier);
				    		
							wc_st_add_tracking_number( $order_id, $label->tracking_reference, $provider, null, null );
						}

						if(get_option("_shipmate_status_success", "Don\'t Change") != "Don\'t Change") {
							if(WooCommerce_Shipmate_Integration::version_check('3.0')) {
								error_log('hit 3');
								$order->set_status(get_option("_shipmate_status_success"));
								$order->save();
							} else {
								error_log('hit 4');
								$order->update_status(get_option("_shipmate_status_success"));
							}
							
						
						} else {
							error_log('something2');
						}
				    }
			    }
	    		
			    update_post_meta( $order_id, $this->printed_meta_key, 'yes' );
			    $message = sprintf( __( 'Shipping Label' . (count($result->data()) == 1 ? '' : 's') . ' printed with Shipmate by %s.', 'shipmate' ), wp_get_current_user()->display_name );
			    $order->add_order_note( $message );
			    echo(json_encode(array('success' => true)));
			    die();
	    		break;
	    	default:
				die();
	    		break;
		}
	}


	function add_buttons($actions) {
		$actions['wc_order_print_label'] = __( 'Print Label with Shipmate', 'shipmate' );
		echo(WooCommerce_Shipmate_Orders_Metabox::output());
		return $actions;
	}

	// function shipmate_process_orders_meta_box() {
	// 	add_meta_box( 'woocommerce-orders-shipmate', __( 'Shipmate Data', 'woocommerce' ) . wc_help_tip( __( 'Print your shipping labels with Shipmate.', 'woocommerce' ) ), 'WooCommerce_Shipmate_Orders_Metabox::output', 'shop_order', 'side', 'default' );
	// 	//add_meta_box(array('WooCommerce_Shipmate_Orders_Metabox::output'));
	// 	//die(include_once( dirname( __FILE__ ) . '/views/html-admin-orders-shipmate.php' ));
	// 	//echo include_once( dirname( __FILE__ ) . '/views/html-admin-orders-shipmate.php' );
	// 	// if( !isset( $_REQUEST['post'] ) && !is_array( $_REQUEST['post'] ) ) {
	// 	// 	return;
	// 	// }

	// 	// foreach( $_REQUEST['post'] as $order_id ) {
	// 	// 	die($order_id);
	// 	// }
	// }

	/**
	 * Add an order note when custom action is clicked
	 * Add a flag on the order to show it's been run
	 *
	 * @param \WC_Order $order
	//  */
	// function shipmate_process_order_meta_box_action( $order ) {
	    
	// 	// attempt to create the parcel inside of Shipmate!
	// 	$post_id = $_POST['post_ID'];
	// 	$order = new WC_Order($post_id);
		
	// 	$consignment = new WooCommerce_Shipmate_Consignment($order, 
	// 		!empty($_POST['parcel_dimensions']) ? $_POST['parcel_dimensions'] : array(), 
	// 		!empty($_POST['delivery_service_key']) ? $_POST['delivery_service_key'] : "", 
	// 		!empty($_POST['dispatch_date']) ? $_POST['dispatch_date'] : date('Y-m-d', strtotime('now'))
	// 	);
	// 	// $consignment->set_delivery_service_key(!empty($_POST['delivery_service_key']) ? $_POST['delivery_service_key'] : "");
		
	// 	$wordpress_user = wp_get_current_user();
	// 	$shipmate_user_id = get_user_meta($wordpress_user->ID, '_shipmate_user_id', true);
	// 	$result = $this->shipmate_service->create_consignment($shipmate_user_id, $consignment->toArray());
		
	// 	switch($result->http_code()) {
	// 		case "403":
	// 			$message = sprintf( __( 'Unable to connect to Shipmate (Invalid Access Token).', 'shipmate' ));
	//     		$order->add_order_note( $message );
	//     		break;
	//     	case "400":
	//     		$message = sprintf( __( 'Unable to create Consignment with Shipmate (Missing Data).', 'shipmate' ));
	//     		$order->add_order_note( $message );


	//     		if(get_option("_shipmate_status_failed", "Don\'t Change") != "Don\'t Change") {
	// 				$order->set_status(get_option("_shipmate_status_failed"));
	// 				$order->save();
	// 			}
	//     		break;
	//     	case "201":
	// 		    foreach($result->data() as $key => $label) {
	// 		    	// if we haven't previously created these parcels, save the tracking reference and dimensions
	// 		    	if(get_post_meta( $order->get_id(), $this->printed_meta_key, true ) != 'yes') {
	// 			    	$message = sprintf( __( 'Shipping Label generated with Tracking Reference %s for %s. (%s - %s)', 'shipmate' ), $label->tracking_reference, $label->parcel_reference, $label->carrier, $label->service_name );
	// 			    	$order->add_order_note( $message );

	// 			    	// store the carrier and service used
	// 			    	update_post_meta( $order->get_id(), '_shipmate_carrier', $label->carrier );
	// 			    	update_post_meta( $order->get_id(), '_shipmate_service', $label->service_name );

	// 			    	// store tracking references / dims in meta for the order
	// 			    	add_post_meta($order->get_id(), '_shipmate_parcel_reference', $label->parcel_reference, false);
	// 			    	add_post_meta($order->get_id(), '_shipmate_tracking_reference', $label->tracking_reference, false);
	// 			    	add_post_meta($order->get_id(), '_shipmate_parcel_weight', $_POST['parcel_dimensions']['parcel_weights'][$key], false);
	// 			    	add_post_meta($order->get_id(), '_shipmate_parcel_length', $_POST['parcel_dimensions']['parcel_lengths'][$key], false);
	// 			    	add_post_meta($order->get_id(), '_shipmate_parcel_width', $_POST['parcel_dimensions']['parcel_widths'][$key], false);
	// 			    	add_post_meta($order->get_id(), '_shipmate_parcel_depth', $_POST['parcel_dimensions']['parcel_depths'][$key], false);

	// 			    	if ( function_exists( 'wc_st_add_tracking_number' ) ) {
	// 			    		$provider = $this->shipmate_integration->convert_carrier_to_provider($label->carrier);

	// 						wc_st_add_tracking_number( $order->get_id(), $label->tracking_reference, $provider, null, null );
	// 					}


	// 					if(get_option("_shipmate_status_success", "Don\'t Change") != "Don\'t Change") {
	// 						$order->set_status(get_option("_shipmate_status_success"));
	// 						$order->save();
	// 					}
	// 			    }
	// 		    }
	    		
	// 		    update_post_meta( $order->get_id(), $this->printed_meta_key, 'yes' );
	// 		    $message = sprintf( __( 'Shipping Label' . (count($result->data()) == 1 ? '' : 's') . ' printed with Shipmate by %s.', 'shipmate' ), wp_get_current_user()->display_name );
	// 		    $order->add_order_note( $message );
	//     		break;
	//     	default:
	//     		die(var_dump($result));
	//     		break;
	// 	}
	// 	wp_redirect($_POST['_wp_http_referer']);
	// }

	// function add_custom_order_status_actions_button( $actions, $order ) {

	// 	if(WooCommerce_Shipmate_Integration::version_check('3.0')) {
	// 		$order_id = $order->get_id();
	// 	} else {
	// 		$order_id = $order->get_id();
	// 	}

	//     // Display the button for all orders that have a 'processing' status
	//     if ( $order->has_status( array( 'processing' ) ) ) {
	//         // The key slug defined for your action button
	//         $action_slug = 'wc_order_print_label';

	//         $action_name = "";
	//         if(get_post_meta( $order_id, $this->printed_meta_key, true ) == 'yes') {
	//         	return $actions;
	// 		    $action_name = __( 'Reprint Label with Shipmate', 'shipmate' );
	// 		} else {
	// 			$action_name = __( 'Print Label with Shipmate', 'shipmate' );
	// 		}

	//         // Set the action button
	//         $actions[$action_slug] = array(
	//             'url'       => wp_nonce_url( admin_url( 'admin-ajax.php?action=woocommerce_order_action_wc_order_print_label&order_id=' . $order_id ), 'woocommerce_order_action_wc_order_print_label' ),
	//             'name'      => $action_name,
	//             'action'    => $action_slug,
	//         );
	//     }
	//     return $actions;
	// }

	// function add_custom_order_status_actions_button_css() {
	//     $action_slug = "wc_order_print_label"; // The key slug defined for your action button

	//     echo '<style>.wc-action-button-'.$action_slug.'::after { font-family: woocommerce !important; content: "\e029" !important; }</style>';
	// }

	// function add_custom_order_status_actions_button_js() {
	// 	echo include_once( dirname( __FILE__ ) . '/views/html-admin-orders-shipmate.php' );
	// }
	/**
	 * Main WooCommerce_Shipmate_Order_Actions Instance
	 *
	 * Ensures only one instance of WooCommerce_Shipmate_Order_Actions is loaded or can be loaded.
	 *
	 * @since 1.0.0
	 * @static
	 * @see WooCommerce_Shipmate()
	 * @return Main WooCommerce_Shipmate_Order_Actions instance
	 */
	public static function instance ( $parent ) {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self( $parent );
		}
		return self::$_instance;
	} // End instance()

	/**
	 * Cloning is forbidden.
	 *
	 * @since 1.0.0
	 */
	public function __clone () {
		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?' ), $this->parent->_version );
	} // End __clone()

	/**
	 * Unserializing instances of this class is forbidden.
	 *
	 * @since 1.0.0
	 */
	public function __wakeup () {
		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?' ), $this->parent->_version );
	} // End __wakeup()

}
