<?php
/**
 * WooCommerce Shipping Settings
 *
 * @author      WooThemes
 * @category    Admin
 * @package     WooCommerce/Admin
 * @version     2.6.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'WC_Settings_Shipmate_Tab', false ) ) :

/**
 * WC_Settings_Shipmate_Tab.
 */
class WC_Settings_Shipmate_Tab extends WC_Settings_Page {

	/**
	 * The shipmate integration class
	 * @var     WooCommerce_Shipmate_Integration
	 * @access  protected
	 * @since   1.0.0
	 */
	protected $shipmate_integration = null;

	/**
	 * Constructor.
	 */
	public function __construct() {
		$this->id    = 'shipmate';
		$this->label = __( 'Shipmate', 'woocommerce' );

		parent::__construct();

		$this->shipmate_integration = new WooCommerce_Shipmate_Integration();
	}

	/**
	 * Add this page to settings.
	 *
	 * @param array $pages
	 *
	 * @return array|mixed
	 */
	public function add_settings_page( $pages ) {
		return parent::add_settings_page( $pages );
	}

	/**
	 * Get sections.
	 *
	 * @return array
	 */
	public function get_sections() {

		// no sections until you are authenticated
		$sections = array();

		if($this->shipmate_integration->is_integrated()) {
			$sections = array(
				''        => __( 'Your Integration', 'woocommerce_shipmate' ),
				'general' => __( 'General Options', 'woocommerce_shipmate' ),
				'users' => __( 'User Mapping', 'woocommerce_shipmate' ),
				'routing' => __( 'Delivery Routing', 'woocommerce_shipmate' ),
				'packaging_options' => __( 'Packaging Options', 'woocommerce_shipmate' )
			);
		}

		return apply_filters( 'woocommerce_get_sections_' . $this->id, $sections );
	}

	/**
	 * Get settings array.
	 *
	 * @param string $current_section
	 *
	 * @return array
	 */
	public function get_settings( $current_section = '' ) {
		if($this->shipmate_integration->is_integrated()) {
			if($current_section == '') {
				return $this->get_integration_details($current_section);
			} else if($current_section == 'users') {
				return $this->get_users_form($current_section);
			} else if($current_section == 'routing') {
				return $this->get_routing_form($current_section);
			} else if($current_section == 'packaging_options') {
				return $this->get_packing_options_form($current_section);
			} else if($current_section == 'general') {
				return $this->get_general_form($current_section);
			}
		} else {
			return $this->get_authentication_form($current_section);
		}
	}

	public function get_packing_options_form($current_section) {
		$settings = array();
		return apply_filters( 'wc_settings_tab_shipmate_settings', $settings );	
	}

	public function get_routing_form($current_section) {
		$settings = array();
		return apply_filters( 'wc_settings_tab_shipmate_settings', $settings );	
	}

	public function get_users_form($current_section) {
		$settings = array();
		return apply_filters( 'wc_settings_tab_shipmate_settings', $settings );	
	}

	public function get_general_form($current_section) {
		$settings = array();
		return apply_filters( 'wc_settings_tab_shipmate_settings', $settings );	
	}

	public function get_integration_details($current_section) {
		$settings = array(
		        'section_title' => array(
		            'name'     => __( 'Your Shipmate Integration', 'woocommerce-settings-tab-shipmate' ),
		            'type'     => 'title',
		            'desc'     => '',
		            'id'       => 'wc_settings_tab_shipmate_section_title'
		        ),
		        'sender_name' => array(
		            'name' => __( 'Sender Name', 'woocommerce-settings-tab-shipmate' ),
		            'type' => 'text',
		            'id'   => 'wc_settings_tab_shipmate_sender_name',
					'default'  => $this->shipmate_integration->sender_name(),
		        ),
		        'api_key' => array(
		            'name' => __( 'Description', 'woocommerce-settings-tab-shipmate' ),
		            'type' => 'text',
		            'id'   => 'wc_settings_tab_shipmate_api_key',
					'default'  => $this->shipmate_integration->token(),
		        ),
		        'section_end' => array(
		             'type' => 'sectionend',
		             'id' => 'wc_settings_tab_shipmate_section_end'
		        )
		    );
	    	return apply_filters( 'wc_settings_tab_shipmate_settings', $settings );	
	}

	public function get_authentication_form($current_section) {
		$settings = array(
		        'section_title' => array(
		            'name'     => __( 'Your Shipmate Account', 'woocommerce-settings-tab-shipmate' ),
		            'type'     => 'title',
		            'desc'     => __( "You have not linked your Shipmate account to this WooCommerce installation.\r\n\r\nSign in using your email address and password and we will generate access tokens for you.\r\n\r\n<b>You must sign in with a Sender account</b>." ),
		            'id'       => 'wc_settings_tab_shipmate_section_title'
		        ),
		        'shipmate_username' => array(
		            'name' => __( 'Username', 'woocommerce-settings-tab-shipmate' ),
		            'type' => 'text',
		            'desc' => __( 'The email address you use to sign in to Shipmate', 'woocommerce-settings-tab-shipmate' ),
		            'id'   => 'wc_settings_shipmate_username'
		        ),
		        'shipmate_password' => array(
		            'name' => __( 'Password', 'woocommerce-settings-tab-shipmate' ),
		            'type' => 'password',
		            'desc' => __( 'The password you use to sign in to Shipmate', 'woocommerce-settings-tab-shipmate' ),
		            'id'   => 'wc_settings_shipmate_password'
		        ),
				array(
		            'name' 			=> 	__( 'Environment', 'woocommerce-settings-tab-shipmate' ),
					'id' 			=> 'wc_settings_shipmate_live',
					'label'			=> __( 'Connect to Live Servers?', 'woocommerce-shipmate' ),
				//	'desc'	=> __( 'Check this box to connect to the live Shipmate environment.', 'woocommerce-shipmate' ),
					'type'			=> 'radio',
					'default'		=> '1',
					'options'     => array (
						'1' => 'Production',
						'0' => 'Staging'
					)
				),
		        'section_end' => array(
		             'type' => 'sectionend',
		             'id' => 'wc_settings_tab_shipmate_section_end'
		        )
		    );

		return apply_filters( 'woocommerce_get_settings_' . $this->id, $settings, $current_section );
	}

	/**
	 * Output the settings.
	 */
	public function output() {
		global $current_section, $hide_save_button;

		if($this->shipmate_integration->is_integrated()) {
			if($current_section == 'users') {
				// custom user form maybe?
				$shipmate_service = new WooCommerce_Shipmate_Service($this->shipmate_integration);
				$users = $shipmate_service->users();

				$shipmate_users = $users->data();
				$wp_users = get_users(array('role__in' => ['administrator', 'shop_manager']));

				include_once( dirname( __FILE__ ) . '/views/html-admin-page-shipmate-users.php' );
				// $settings = $this->get_settings($current_section);
				// WC_Admin_Settings::output_fields( $settings );
			} else if ($current_section == 'routing') {
				
				// get shipping options
				$wc_shipping = WC_Shipping::instance();
				$wp_shipping_options = $wc_shipping->get_shipping_methods();

				include_once( dirname( __FILE__ ) . '/views/html-admin-page-shipmate-routing.php' );
			} else if($current_section == 'packaging_options') {

				$shipmate_service = new WooCommerce_Shipmate_Service($this->shipmate_integration);
				$options = $shipmate_service->packaging_option();

				$packaging_option = $options->data();


				include_once( dirname( __FILE__ ) . '/views/html-admin-page-shipmate-packaging-options.php' );
			} else if ($current_section == 'general') {
				
				include_once( dirname( __FILE__ ) . '/views/html-admin-page-shipmate-general.php' );
			} else {
				// display integration information (token and sender name)
				$hide_save_button = true;
				include_once( dirname( __FILE__ ) . '/views/html-admin-page-shipmate-authenticated.php' );
			}
		} else {
			$settings = $this->get_settings($current_section);
			WC_Admin_Settings::output_fields( $settings );
		}
	}

	/**
	 * Save settings.
	 */
	public function save() {
		global $current_section;

		if($this->shipmate_integration->is_integrated()) {
			if($current_section == 'users') {
				// we need to connect users to each other
				$this->save_users_form();
			} else if ($current_section == 'routing') {
				$this->save_routing_form();
			} else if ($current_section == "packaging_options") {
				//Packaging Options is fully controlled in the Shipmate system so we don't need to save anything.
			} else if ($current_section == "general") {
				$this->save_general_form();
			} else {
				// do nothing
				$this->disconnect_integration();
			}
		} else {
			$this->save_authentication_form();
		}
	}

	public function disconnect_integration() {
		$shipmate_service = new WooCommerce_Shipmate_Service($this->shipmate_integration);
		$shipmate_service->disconnect();

		wp_redirect(admin_url('/admin.php?page=wc-settings&tab=shipmate&section', 'http'));
	}


	public function save_general_form() {
		$shipmate_process_block = $_POST['shipmate_process_block'];
		update_option("_shipmate_process_block", $shipmate_process_block);

		$shipmate_dimensions_unit = $_POST['shipmate_dimensions_unit'];
		update_option("_shipmate_dimensions_unit", $shipmate_dimensions_unit);

		$shipmate_weights_unit = $_POST['shipmate_weights_unit'];
		update_option("_shipmate_weights_unit", $shipmate_weights_unit);

		$shipmate_success_status = $_POST['shipmate_success_status'];
		update_option("_shipmate_status_success", $shipmate_success_status);

		$shipmate_failed_status = $_POST['shipmate_failed_status'];
		update_option("_shipmate_status_failed", $shipmate_failed_status);

		$shipmate_cancel_status = $_POST['shipmate_cancel_status'];
		update_option("_shipmate_status_cancel", $shipmate_cancel_status);
	}

	public function save_routing_form() {
		$shipping_options = $_POST['shipping_option'];
		$delivery_services = $_POST['delivery_service'];

		foreach($shipping_options as $key => $option) {
			update_option("_shipmate_delivery_service_" . $key, $delivery_services[$key]);
		}
	}

	public function save_users_form() {
		if(!empty($_POST['users'])) {
			foreach($_POST['users'] as $wp_user_id => $shipmate_user_id) {
				update_user_meta($wp_user_id, '_shipmate_user_id', $shipmate_user_id);
			}
		}
	}

	public function save_authentication_form() {
		$shipmate_service = new WooCommerce_Shipmate_Service($this->shipmate_integration);
		
		$username 	= isset($_POST['wc_settings_shipmate_username']) ? $_POST['wc_settings_shipmate_username'] : "";
		$password 	= isset($_POST['wc_settings_shipmate_password']) ? $_POST['wc_settings_shipmate_password'] : "";
		$url 		= admin_url();
		$live		= isset($_POST['wc_settings_shipmate_live']) ? $_POST['wc_settings_shipmate_live'] : "";

		// check set correctly, give validation errors if not
		$errors = array();
		if(empty($username)) { $errors['wc_settings_shipmate_username'] = true; }
		if(empty($password)) { $errors['wc_settings_shipmate_password'] = true; }
		if(empty($url)) { $errors['wc_settings_shipmate_url'] = true; }

		if(!empty($errors)) {
			// Bad! Do something.
			return;
		}

		// hit the shipmate api to create the channel integration
		$result = $shipmate_service->connect($_POST);
		// die(Var_dump($result));
		if($result->http_code() != 200) {
			// BAD - can we return some sort of flash message?
			return;
		}

		// store token / api_key / url
		$this->shipmate_integration->save($result->data()->api_key, $result->data()->name, $live ? "1" : "0");
		wp_redirect(admin_url('/admin.php?page=wc-settings&tab=shipmate&section', 'http'));
	}
}

endif;

return new WC_Settings_Shipmate_Tab();
